include asm.e
include ports.e


constant no_fading=-666
constant no_delay=-667

constant vfix=3.14159265/180
constant zEye=-200


sequence inRegs,outRegs
global sequence m19_centers,m19_orderTb
sequence cosTb, sinTb

sinTb=repeat(0,360)
cosTb=sinTb
for i=1 to 360 do
 sinTb[i]=sin(i*vfix)
 cosTb[i]=cos(i*vfix)
end for


--** <Assembly code> **--

--*** Copy one screen to another ***--
global constant show_buffer_code=get_asm(
"pusha cld "&
"mov edi,655360 "&		--address of vga memory
"mov esi,copy_src_addr "&
"mov ecx,16000 "&
"rep movsd "&
"popa ret ")
global constant copy_src_addr=show_buffer_code+get_param("copy_src_addr")

--*** Wait for a vertical retrace ***--
global constant wait_vrt_code=get_asm(
"pusha cld "&
"mov edx,986 "&
"waitLbl1: "&
"in ax,dx "&
"and al,8 "&
"jnz waitLbl1 "&
"waitLbl2: "&
"in ax,dx "&
"and al,8 "&
"jz waitLbl2 "&
"popa ret")

--** </Assembly code> **--


--** <Misc stuff> **--

--*** Set mode 19 ***--
global procedure set_vga()
outRegs={0,0,0,0,0,0,0,0,0,0}
outRegs[REG_AX]=#13
inRegs=dos_interrupt(#10,outRegs)
end procedure

--*** Restore graphics mode and free some memory ***--
global procedure shut_down()
 if machine_func(5,-1) then end if
 free(asm_xmin)
 free(asm_xmax)
 free(asm_cmin)
 free(asm_cmax)
 free(asm_umin)
 free(asm_umax)
 free(asm_vmin)
 free(asm_vmax)
 free(ghl_code)
 free(thl_code)
 free(thl_deltaX)
 free(show_buffer_code)
 free(wait_vrt_code)
 free(sgps_code)
 free(sgps_data)
 free(stps_code)
 free(stps_data)
 free(reset_minmax_buffers)
end procedure


--** poke a bitmap into memory **--
global function poke_bitmap(sequence tha_bgf_img)
sequence ret_seq
atom tha_mem_space
 tha_mem_space=allocate((length(tha_bgf_img)*length(tha_bgf_img[1])))
 ret_seq=repeat(0,3)
 for i=1 to length(tha_bgf_img) do
  poke(tha_mem_space+(length(tha_bgf_img[1])*(i-1)),tha_bgf_img[i])
 end for
 ret_seq[1]=tha_mem_space
 ret_seq[2]=length(tha_bgf_img[1])
 ret_seq[3]=length(tha_bgf_img)
 return ret_seq
end function


--** copy a virtual screen to vga memory **--
global procedure asm_show_buffer(atom src)
 poke4(copy_src_addr,src)
 call(show_buffer_code)
end procedure

--** clear a screen **--
global procedure mem_cls(atom tha_screen_addr,integer tha_col)
 mem_set(tha_screen_addr,tha_col,64000)
end procedure

--** wait for a vertical retrace **--
global procedure wait_vrt()
call(wait_vrt_code)
end procedure

--** set the specified palette **--
global procedure set_pal(sequence tha_pal)
for i=1 to 256 do
Output(i-1,#3C8)
Output(tha_pal[i][1],#3C9)
Output(tha_pal[i][2],#3C9)
Output(tha_pal[i][3],#3C9)
end for
end procedure

--** </Misc stuff> **--


--** <Math routines> **--

--*** Return the absolute value of num1 ***--
global function m19abs(integer num1)
if (num1<0) then
num1=-num1
end if
return num1
end function

--*** Return the differance between two integers ( diff(-1,2)=3 ) ***--
global function diff(integer num1,integer num2)
integer retNum
if (num1>=0) and (num2>=0) then
retNum=m19abs(num1-num2)
end if
if (num1>=0) and (num2<0) then
retNum=num1+m19abs(num2)
end if
if (num1<0) and (num2>=0) then
retNum=m19abs(num1)+num2
end if
if (num1<0) and (num2<0) then
retNum=m19abs(num1-num2)
end if
return retNum
end function


--*** Convert 3d-coordinates to 2d ***--
global function get_screen_xy(sequence xy,sequence obj)
integer x0,y0
integer numFaces,facePoints
sequence retSeq
 numFaces=length(obj)
 facePoints=length(obj[1])
 retSeq=repeat(repeat({0,0},facePoints),numFaces)
 for i=1 to numFaces do
  for j=1 to facePoints do
   x0=xy[1]+floor(obj[i][j][1]*(zEye/(zEye-obj[i][j][3])))
   y0=xy[2]+floor(obj[i][j][2]*(zEye/(zEye-obj[i][j][3])))
   retSeq[i][j]={x0,y0}
  end for
 end for
 return retSeq
end function



--*** Sort an array of faces ***--
global function sort_faces(sequence obj)
integer pos
atom tempval
integer numFaces,facePoints
 facePoints=length(obj[1])
 numFaces=length(obj)
 m19_centers=repeat(0,numFaces)
 m19_orderTb=m19_centers
 for i=1 to numFaces do
  for j=1 to facePoints do
   m19_centers[i]=m19_centers[i]+obj[i][j][3]
  end for
 end for
 for i=1 to numFaces do
  m19_orderTb[i]=i
 end for
 pos=1
 for i=1 to numFaces do
  for j=(i+1) to numFaces do
   if m19_centers[j]>m19_centers[i] then
    tempval=m19_centers[j]
    m19_centers[j]=m19_centers[i]
    m19_centers[i]=tempval
    tempval=m19_orderTb[j]
    m19_orderTb[j]=m19_orderTb[i]
    m19_orderTb[i]=tempval
   end if
  end for
 end for
return m19_orderTb
end function


--*** Rotate a sequence of points ***--
global function rotate(sequence obj,sequence toRotate,sequence a)
atom xr,yr,zr
integer objPoints
 objPoints=length(obj)
 for i=1 to objPoints do
  if toRotate[1]=1 then
   yr=(cosTb[a[1]]*obj[i][2])-(sinTb[a[1]]*obj[i][3])
   zr=(sinTb[a[1]]*obj[i][2])+(cosTb[a[1]]*obj[i][3])
   obj[i][2..3]={yr,zr}
  end if
  if toRotate[2]=1 then
   xr=(cosTb[a[2]]*obj[i][1])-(sinTb[a[2]]*obj[i][3])
   zr=(sinTb[a[2]]*obj[i][1])+(cosTb[a[2]]*obj[i][3])
   obj[i][1]=xr
   obj[i][3]=zr
  end if
  if toRotate[3]=1 then
   xr=(cosTb[a[3]]*obj[i][1])-(sinTb[a[3]]*obj[i][2])
   yr=(sinTb[a[3]]*obj[i][1])+(cosTb[a[3]]*obj[i][2])
   obj[i][1..2]={xr,yr}
  end if
 end for
 return obj
end function


global function get_quad_zNormal(sequence d3Coords)
sequence retNorm
integer numFaces
 numFaces=length(d3Coords) 
 retNorm=repeat(0,numFaces)
 for i=1 to numFaces do
--  retNorm[i]=(d3Coords[i][1][3]+d3Coords[i][3][3])
  retNorm[i]=(d3Coords[i][1][3]+d3Coords[i][2][3]+d3Coords[i][3][3]+d3Coords[i][4][3])
  --retNorm[i]=(retNorm[i]/m19abs(retNorm[i]))
 end for
return retNorm
end function

--** </Math routines> **--

